package com.zanxiang.sdk.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.CallBackEnum;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.*;
import com.zanxiang.mybatis.mapper.OrderMapper;
import com.zanxiang.sdk.domain.dto.GamePayWayDTO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.domain.params.ProductPayParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * 平台订单逻辑开发
 *
 * @author xufeng
 * @date 2022-06-06 17:06
 */
@Slf4j
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {

    @Autowired
    private IPromoChannelService promoChannelService;

    @Autowired
    private IGameService gameService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IGameUserService gameUserService;

    @Autowired
    private IGameUserRoleService gameUserRoleService;

    @Autowired
    private IGamePayWayService gamePayWayService;

    @Override
    @Transactional(rollbackFor = {Exception.class, RuntimeException.class})
    public Boolean createOrder(ProductPayParam payParam, UserData userData) {
        //用户id
        Long userId = userData.getUserId();
        //游戏id
        Long gameId = userData.getGameId();
        //游戏信息
        Game game = gameService.getById(userData.getGameId());
        if (game == null) {
            throw new BaseException("参数错误, 游戏信息不存在");
        }
        //用户信息
        User user = userService.getById(userId);
        if (user == null) {
            throw new BaseException("参数错误, 用户信息不存在");
        }
        //玩家信息
        GameUser gameUser = gameUserService.getOne(new LambdaQueryWrapper<GameUser>()
                .eq(GameUser::getUserId, userData.getUserId()));
        if (gameUser == null) {
            throw new BaseException("参数错误, 玩家信息不存在");
        }
        //角色信息
        GameUserRole gameUserRole = gameUserRoleService.getOne(new LambdaQueryWrapper<GameUserRole>()
                .eq(GameUserRole::getGameId, userData.getGameId())
                .eq(GameUserRole::getUserId, userData.getUserId())
                .eq(GameUserRole::getRoleId, payParam.getRoleId()));
        if (gameUserRole == null) {
            throw new BaseException("参数错误, 角色信息不存在");
        }
        //游戏支付配置信息
        GamePayWayDTO gamePayWayDTO = gamePayWayService.getGamePayWay(gameId, payParam.getPayWay(), payParam.getPayDevice());
        //生成订单id
        String orderNum = this.getOrderNum(userData.getUserId());
        //构造订单
        super.save(Order.builder()
                .orderId(orderNum)
                .agentId(promoChannelService.getAgentIdByChannel(userData.getChannel()))
                .cpId(game.getCpId())
                .cpOrderId(payParam.getOrderId())
                .userId(userData.getUserId())
                .mgUserId(gameUser.getId())
                .roleId(gameUserRole.getRoleId())
                .roleName(gameUserRole.getRoleName())
                .serverId(gameUserRole.getServerId())
                .serverName(gameUserRole.getServerName())
                .gameId(userData.getGameId())
                .amount(payParam.getAmount())
                .productId(payParam.getProductId())
                .productName(payParam.getProductName())
                .status(OrderStateEnum.READY_PAY.getCode())
                .gamePayWayId(gamePayWayDTO.getId())
                .ext(payParam.getExtension())
                .payDeviceId(gamePayWayDTO.getPayDeviceId())
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .username(user.getUsername())
                .regTime(user.getCreateTime())
                .deviceSystem(userData.getDeviceSystem())
                .payWayId(gamePayWayDTO.getPayWayId())
                .regGameId(user.getGameId())
                .callBackStatus(CallBackEnum.UN_CALL_BACK.getCallBackStatus())
                .roleLevel(gameUserRole.getRoleLevel())
                .roleVipLevel(gameUserRole.getRoleVipLevel())
                .build());
        //设置订单号
        payParam.setOrderId(orderNum);
        //返回
        return Boolean.TRUE;
    }

    private String getOrderNum(Long userId) {
        //时间（精确到毫秒）
        DateTimeFormatter ofPattern = DateTimeFormatter.ofPattern("yyyyMMddHHmmssSSS");
        String localDate = LocalDateTime.now().format(ofPattern);
        //3位随机数
        String randomNumeric = RandomStringUtils.randomNumeric(3);
        //5位用户id
        int subStrLength = 5;
        String sUserId = userId.toString();
        int length = sUserId.length();
        String str;
        if (length >= subStrLength) {
            str = sUserId.substring(length - subStrLength, length);
        } else {
            str = String.format("%0" + subStrLength + "d", userId);
        }
        return localDate + str + randomNumeric;
    }

    @Override
    public PlatformOrderDTO getByOrderId(String orderId) {
        Order order = getOne(new LambdaQueryWrapper<Order>().eq(Order::getOrderId, orderId));
        return BeanUtils.copy(order, PlatformOrderDTO.class);
    }
}
